# main.s: This file contains the main procedure and is a basis
# for the higher levels of code. The main procedure
# is called after initialisation is complete.

# Global procedures and functions defined in this file:
.globl main

# Global procedures and functions called in this file:
.globl receive_character
.globl display_result
.globl transmit_string
.globl toggle_input
.globl driver
.globl calibrate_0
.globl calibrate_180
.globl diagnostic
.globl debug_print_out_buffer
.globl version_string


                
# main: Main program procedure. (No parameters)
main:           # On bootup, print out the version number
                ld          bc,version_string
                push        bc
                call        transmit_string
                pop         bc

main_loop:      
				# Display the latest position calculated by the system.
                ld			b,0		# current result number
                push		bc
				call		display_result
				pop			af

				# Wait for the user to press a key on the keypad
spin_for_character:
                push        af
                call        receive_character
                pop         af

				# receive_character returns zero if there has been
				# no input.
                cp          0
                jr          z,spin_for_character

				# Do a fast position acquire when the user presses 1..
                cp          '1'
				jr			z,do_a_scan_fast

				# Do a slow position acquire when the user presses 1..
                cp          '2'
				jr			z,do_a_scan_slow


				# When the user presses 3, he gets the infra-red
				# diagnostic utility.
                cp          '3'
                call         z,diagnostic

				# When the user presses A, the next result (if any)
				# appears on screen
                cp          'a'
                jr          z,next_result

				# When the user presses B, an earlier result (if any)
				# appears on screen
                cp          'b'
                jr          z,previous_result

                # Hidden serial line features.
				# When the user presses P, print out the scan buffer
                cp          'p'
                call        z,debug_print_out_buffer

				# When the user presses K, he is presented with
				# the calibration menu, allowing the zero degree
				# position of the scanner to be calibrated.
                cp          'k'
                call         z,calibrate_0

				# When the user presses L, he gets the calibration
				# menu for 180 degrees.
                cp          'l'
                call         z,calibrate_180

				# Pressing Escape (only possible via the serial line)
				# causes the program to jump to address zero. This will
				# do a soft reset of the system.
                cp          27
                jp          z,0

                push		bc
				call		display_result
				pop			af

                jr          spin_for_character

do_a_scan_fast: ld          a,1
                jr          do_scan

do_a_scan_slow: ld          a,0

do_scan:
                push        af
               	# Before calling driver, disable input.
                # We don't want the user to press anything
                # while the driver is running, since the
				# keypad/serial handling ISR may interfere
				# with the timing of the servo.

                ld          a,0
                push        af
                call        toggle_input
                pop         af

                call		driver

                # Re-enable input:
                inc         a       # set A = 1
                push        af
                call        toggle_input
                pop         af

                pop         af

				# And go back to the main loop
				ld			b,0		# reset current result number
				jr			spin_for_character


previous_result:
				ld			c,b		# save current result number
				inc			b
				jr			change_result

next_result:	ld			c,b		# save current result number
				dec			b		# decrease it

change_result:
				push		bc
				call		display_result
				pop			af

				# if A = 0, then the result requested was not
				# available, and we restore the old result number
				# from C. Otherwise, we carry on.
				cp			0
				jr			nz,spin_for_character

				ld			b,c		# restore old value.
				jr			spin_for_character



